SystemOrganization addCategory: #'Seaside-FileSystem'!

WAEntryPoint subclass: #WAExternalFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibrary class methodsFor: 'accessing' stamp: 'lr 2/10/2008 14:49'!
description
	^ 'File Directory'! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:26'!
asAbsoluteUrl: aFileName relativeTo: aRequest
	^aRequest url copy
		addToPath: aFileName;
		yourself! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 4/20/2008 21:18'!
contentsOfFile: aFilename
	| stream |
	stream := aFilename readStream.
	^[
		stream binary.
		stream upToEnd ]
		ensure: [ stream close ]! !

!WAExternalFileLibrary methodsFor: 'configuration' stamp: 'pmm 10/18/2008 10:56'!
defaultConfiguration
	^ super defaultConfiguration
		addParent: WAExternalFileLibraryConfiguration instance;
		yourself! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
directory
	| path filename |
	path := self preferenceAt: #directory.
	filename := SpFilename named: path.
	"hack around SPort bugs"
	^(path last = filename separator)
		ifTrue: [ SpFilename named: path allButLast ]
		ifFalse: [ filename ]! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:33'!
filenameForRequest: aRequest
	| filename basePathUrl |
	filename := self directory.
	basePathUrl := WAUrl new parsePath: self basePath; yourself.
	(basePathUrl relativeTo: aRequest url) do: [ :each |
		filename := filename construct: each ].
	^filename! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 4/20/2008 20:50'!
filesIn: aPath do: aOneArgumentBlock
	"this can not be done with SPort"
	(SeasidePlatformSupport filesIn: aPath) do: [ :each |
		aOneArgumentBlock value: (SeasidePlatformSupport localNameOf: each) ]! !

!WAExternalFileLibrary methodsFor: 'processing' stamp: 'pmm 10/18/2008 11:23'!
handleRequest: aRequestContext
	^ [ self processRequest: aRequestContext request ]
		on: Error 
		do: [ :err | WAResponse notFound: aRequestContext request url ]! !

!WAExternalFileLibrary methodsFor: 'accessing' stamp: 'pmm 10/18/2008 11:10'!
listing
	^ self preferenceAt: #listing! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 12:09'!
listing: aString forRequest: aRequest
	^ WARenderCanvas builder
		rootBlock: [ :html | html title: aRequest url seasideString ];
		renderResponse: [ :html |
			html heading
				level: 1;
				with: aRequest url.
			html unorderedList: [
				self filesIn: aString do: [ :each | 
					html listItem: [
						html anchor
							url: (self asAbsoluteUrl: each relativeTo: aRequest);
							with: each ] ] ] ]! !

!WAExternalFileLibrary methodsFor: 'private' stamp: 'pmm 10/18/2008 11:30'!
mimetypeForRequest: aRequest
	| path |
	path := aRequest url path.
	^(path isEmpty or: [ (path last includes: $.) not ])
		ifTrue: [ WAFileLibrary defaultMimeType seasideMimeType ]
		ifFalse: [
			| extension |
			extension := path last copyAfterLast: $..
			WAFileLibrary mimetypeFor: extension ]! !

!WAExternalFileLibrary methodsFor: 'processing' stamp: 'pmm 3/21/2008 17:33'!
processDirectory: aString request: aRequest
	^ self listing: aString forRequest: aRequest! !

!WAExternalFileLibrary methodsFor: 'processing' stamp: 'pmm 10/18/2008 12:14'!
processFile: aFilename request: aRequest
	^ WAResponse 
		document: (self contentsOfFile: aFilename)
		mimeType: (self mimetypeForRequest: aRequest)
		fileName: aRequest url path last! !

!WAExternalFileLibrary methodsFor: 'processing' stamp: 'pmm 10/18/2008 12:10'!
processRequest: aRequest
	| filename |
	filename := self filenameForRequest: aRequest.
	^ (self listing and: [ filename isDirectory ])
		ifTrue: [ self processDirectory: filename asString request: aRequest ]
		ifFalse: [ self processFile: filename request: aRequest ]! !

WASystemConfiguration subclass: #WAExternalFileLibraryConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-FileSystem'!

!WAExternalFileLibraryConfiguration methodsFor: 'attributes' stamp: 'pmm 10/18/2008 11:12'!
describeOn: config
	(config string: #directory)
		label: 'Directory';
		comment: 'The base directory to serve files from.';
		default: (SpFilename named: 'xxxDoesNotExists') asAbsoluteFilename directory asString.
	(config boolean: #listing)
		label: 'Allow Listing';
		comment: 'Enable listing of directory entries.';
		default: false! !
